//sources:
//https://developer.android.com/samples/FingerprintDialog/index.html
//https://www.androidauthority.com/how-to-add-fingerprint-authentication-to-your-android-app-747304/

/**
 * This class is a callback for the FingerprintManager
 */

package com.caterpillarEffect.cmsc435.authentication_prototypes;

import android.Manifest;
import android.app.Activity;
import android.content.Context;
import android.content.Intent;
import android.content.pm.PackageManager;
import android.hardware.fingerprint.FingerprintManager;
import android.os.CancellationSignal;
import android.support.v4.app.ActivityCompat;
import android.widget.Toast;

public class FingerprintHelper extends FingerprintManager.AuthenticationCallback {

    private static final String TAG = "FingerprintHelper ";

    private final FingerprintManager mFingerprintManager;
    private CancellationSignal mCancellationSignal;
    private Context context;
    private Context packageContext;
    private Activity fingerprintIntent;
    private boolean isLast;
    private long sessionID;
    private FingerPrintAuth fingerprintAuth;

    public FingerprintHelper(FingerprintManager fingerprintManager, Context mContext,
                             Context packageContext, Activity fingerprintIntent, boolean isLast
            , long sessionID, FingerPrintAuth fingerprintAuth) {

        mFingerprintManager = fingerprintManager;
        context = mContext;
        this.packageContext = packageContext;
        this.fingerprintIntent = fingerprintIntent;
        this.isLast = isLast;
        this.sessionID = sessionID;
        this.fingerprintAuth = fingerprintAuth;
    }

    public boolean isFingerprintAuthAvailable() {
        // The line below prevents the false positive inspection from Android Studio
        // noinspection ResourceType
        return mFingerprintManager.isHardwareDetected()
                && mFingerprintManager.hasEnrolledFingerprints();
    }

    /**
     * Starts listening for fingerprint scan
     */
    public void startListening(FingerprintManager.CryptoObject cryptoObject) {
        if (!isFingerprintAuthAvailable()) {
            return;
        }

        mCancellationSignal = new CancellationSignal();
        if (ActivityCompat.checkSelfPermission(context,
                Manifest.permission.USE_FINGERPRINT) != PackageManager.PERMISSION_GRANTED) {
            return;
        }
        mFingerprintManager.authenticate(cryptoObject, mCancellationSignal, 0, this, null);
    }

    /**
     * This method checks to see if FingerprintManager is listening for fingerprint scan
     */
    public boolean isListening() {
        return (mCancellationSignal != null);
    }

    /**
     * This method will stop the fingerprintManager from listening
     */
    public void stopListening() {
        if(isListening()) {
            mCancellationSignal.cancel();
            mCancellationSignal = null;
        }
    }

    @Override
    public void onAuthenticationError(int errMsgId,
                                      CharSequence errString) {
        Toast.makeText(context,
                errString,
                Toast.LENGTH_LONG).show();
    }

    /**
     * When the authentication fails, collect analytics about the failure
     */
    @Override
    public void onAuthenticationFailed() {
        Toast.makeText(context,
                "Authentication failed, please try again ",
                Toast.LENGTH_LONG).show();

        //If this action is the first action the user took,
        //then save the time the user took to take this action
        if(FingerPrintAuth.isFirstAction) {
            FingerPrintAuth.isFirstAction = false;
            FingerPrintAuth.firstActionTime = System.currentTimeMillis() - FingerPrintAuth.timeStarted;
        }

        FingerPrintAuth.TOTAL_CLICKS++;
        FingerPrintAuth.numFailedAuthentication++;
    }

    /**
     * This method gets callback when a user scans their fingerprint wrong.
     * The device will tell the user how to scan their fingerprint.
     */
    @Override
    public void onAuthenticationHelp(int helpMsgId,
                                     CharSequence helpString) {
        Toast.makeText(context,
                 helpString,
                Toast.LENGTH_LONG).show();

        //If this action is the first action the user took,
        //then save the time the user took to take this action
        if(FingerPrintAuth.isFirstAction) {
            FingerPrintAuth.isFirstAction = false;
            FingerPrintAuth.firstActionTime = System.currentTimeMillis() - FingerPrintAuth.timeStarted;
        }

        FingerPrintAuth.TOTAL_CLICKS++;
        FingerPrintAuth.numFailedAuthentication++;
    }


    /**
     * If the authentication is a success, then save and post the analytics to the web-page
     */
    @Override
    public void onAuthenticationSucceeded(
            FingerprintManager.AuthenticationResult result) {

        FingerPrintAuth.TOTAL_CLICKS++;

        //If this action is the first action the user took,
        //then save the time the user took to take this action
        if(FingerPrintAuth.isFirstAction) {
            FingerPrintAuth.isFirstAction = false;
            FingerPrintAuth.firstActionTime = System.currentTimeMillis() - FingerPrintAuth.timeStarted;
        }
        Toast.makeText(context,
                "Authentication Success!",
                Toast.LENGTH_LONG).show();
        stopListening();
        FingerPrintAuth.numSuccessAuthentication++;


        Intent loggedInActivity = new Intent(packageContext, LoggedInActivity.class);
        loggedInActivity.putExtra(MainActivity.IS_LAST, isLast);
        loggedInActivity.putExtra(MainActivity.SESSION_ID, sessionID);
        fingerprintAuth.postAnalytics();
        fingerprintIntent.startActivityForResult(loggedInActivity, MainActivity.REQUEST_CODE);
    }
}
